// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"

namespace Dawn {
  class PokerPlayer;

  enum PokerTurnType {
    POKER_TURN_TYPE_OUT,
    POKER_TURN_TYPE_FOLD,
    POKER_TURN_TYPE_BET,
    POKER_TURN_TYPE_CALL,
    POKER_TURN_TYPE_CHECK,
    POKER_TURN_TYPE_ALL_IN
  };

  struct PokerTurn {
    public:
      /** What type of action the turn is */
      enum PokerTurnType type;
      /** How many chips they did in their turn (if applicable) */
      int32_t chips;
      /** How confident the AI is about their turn. 0 = none, 1 = full */
      float_t confidence;
      /** Player that this action belongs to */
      PokerPlayer *player;

      /**
       * Generate a turn action for betting as a player.
       * 
       * @param player Player index who is betting.
       * @param chips Chips to raise by.
       * @return A turn for a bet action.
       */
      static struct PokerTurn bet(PokerPlayer *player, int32_t chips);

      /**
       * Return a turn action for the given player to fold.
       * 
       * @return A turn for a fold action.
       */
      static struct PokerTurn fold(PokerPlayer *player);

      /**
       * Actions / Performs this turn against the defined player.
       */
      void action();
  };
}