// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PokerWinning.hpp"
#include "PokerPlayer.hpp"

using namespace Dawn;

float_t PokerWinning::getWinningTypeConfidence(enum PokerWinningType type) {
  switch(type) {
    case POKER_WINNING_TYPE_ROYAL_FLUSH:
      return POKER_WINNING_CONFIDENCE_ROYAL_FLUSH;
    case POKER_WINNING_TYPE_STRAIGHT_FLUSH:
      return POKER_WINNING_CONFIDENCE_STRAIGHT_FLUSH;
    case POKER_WINNING_TYPE_FOUR_OF_A_KIND:
      return POKER_WINNING_CONFIDENCE_FOUR_OF_A_KIND;
    case POKER_WINNING_TYPE_FULL_HOUSE:
      return POKER_WINNING_CONFIDENCE_FULL_HOUSE;
    case POKER_WINNING_TYPE_FLUSH:
      return POKER_WINNING_CONFIDENCE_FLUSH;
    case POKER_WINNING_TYPE_STRAIGHT:
      return POKER_WINNING_CONFIDENCE_STRAIGHT;
    case POKER_WINNING_TYPE_THREE_OF_A_KIND:
      return POKER_WINNING_CONFIDENCE_THREE_OF_A_KIND;
    case POKER_WINNING_TYPE_TWO_PAIR:
      return POKER_WINNING_CONFIDENCE_TWO_PAIR;
    case POKER_WINNING_TYPE_PAIR:
      return POKER_WINNING_CONFIDENCE_PAIR;
    default:
      return POKER_WINNING_CONFIDENCE_HIGH_CARD;
  }
}

struct Card PokerWinning::compare(
  struct PokerWinning *left,
  struct PokerWinning *right
) {
  assertNotNull(left);
  assertNotNull(right);

  uint8_t i;
  enum CardValue number = CARD_VALUE_INVALD;
  enum CardValue highNumberLeft = CARD_VALUE_INVALD;
  enum CardValue highNumberRight = CARD_VALUE_INVALD;
  struct Card card(0xFF), highCardLeft(0xFF), highCardRight(0xFF);
  int32_t index;
  uint8_t countCardsSame;

  countCardsSame = 0;

  for(i = 0; i < left->set.size(); i++) {
    card = left->set[i];
    number =  card.getValue();
    // Quick check
    if(highNumberLeft != CARD_VALUE_INVALD && number < highNumberLeft) continue;

    // Check if this number is within the other hand or not
    index = Card::containsNumber(&right->set, number);
    if(index != -1) {
      // This number IS within the other hand, let's check that the EXACT card
      // is a match/isn't a match.
      index = Card::contains(&right->set, card);

      // Exact card match
      if(index != -1) {
        countCardsSame++;
        continue;
      }
      // Not exact card match.. ?
    }

    if(
      highNumberLeft == CARD_VALUE_INVALD ||
      number == CARD_ACE ||
      highNumberLeft < number
    ) {
      highNumberLeft = number;
      highCardLeft = card;
    }
  }
  
  for(i = 0; i < right->set.size(); i++) {
    card = right->set[i];
    number = card.getValue();
    if(highNumberRight != CARD_VALUE_INVALD && number < highNumberRight) {
      continue;
    }
    
    index = Card::containsNumber(&left->set, number);
    if(index != -1) {
      index = Card::contains(&left->set, card);
      if(index != -1) continue;
    }

    if(
      highNumberRight == CARD_VALUE_INVALD ||
      number == CARD_ACE || highNumberRight < number
    ) {
      highNumberRight = number;
      highCardRight = card;
    }
  }


  if(countCardsSame == left->set.size()) {
    for(i = 0; i < left->set.size(); i++) {
      card = left->set[i];
      number = card.getValue();
      if(
        highNumberLeft == CARD_VALUE_INVALD ||
        number == CARD_ACE ||
        highNumberLeft < number
      ) {
        highNumberLeft = number;
        highCardLeft = card;
      }
    }
    return highCardLeft;
  }

  if(highCardLeft.cardValue == CARD_VALUE_INVALD) return 0xFF;
  if(highNumberLeft < highNumberRight) return 0xFF;
  return highCardLeft;// Greater or Equal to.
}

void PokerWinning::fillRemaining() {
  uint8_t i, highest, current;
  struct Card highestCard(0x00);
  struct Card currentCard(0x00);

  // Set the kicker
  this->kicker = 0xFF;

  // Fill the remaining cards
  while(this->set.size() < POKER_WINNING_SET_SIZE) {
    highest = 0xFF;

    for(i = 0; i < this->full.size(); i++) {
      currentCard = this->full[i];
      if(Card::contains(&this->set, currentCard) != -1) continue;

      if(highest == 0xFF) {
        highestCard = currentCard;
        highest = highestCard.getValue();
      } else {
        current = currentCard.getValue();
        if(current != CARD_ACE && current < highest) continue;
        highestCard = currentCard;
        highest = current;
      }
    }

    if(highest == 0xFF) break;
    this->set.push_back(highestCard);
  }
  Card::sort(&this->set);
}